package com.ruoyi.common.utils;

import com.ruoyi.common.constant.HttpStatus;
import com.ruoyi.common.core.domain.entity.DataSource;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.exception.tenantdatassource.TenantDataSource;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;

/**
 * 安全服务工具类
 *
 * @author ruoyi
 */
public class SecurityUtils {

    /**
     * 用户ID
     **/
    public static Long getUserId() {
        try {
            return getLoginUser().getUserId();
        } catch (Exception e) {
            throw new ServiceException("获取用户ID异常", HttpStatus.UNAUTHORIZED);
        }
    }

    /**
     * 获取部门ID
     **/
    public static Long getDeptId() {
        try {
            return getLoginUser().getDeptId();
        } catch (Exception e) {
            throw new ServiceException("获取部门ID异常", HttpStatus.UNAUTHORIZED);
        }
    }

    /**
     * 获取用户账户
     **/
    public static String getUsername() {
        try {
            return getLoginUser().getUsername();
        } catch (Exception e) {
            throw new ServiceException("获取用户账户异常", HttpStatus.UNAUTHORIZED);
        }
    }

    public static String getUserType() {
        try {
            return getLoginUser().getUserType();
        } catch (Exception e) {
            throw new ServiceException("获取用户账户异常", HttpStatus.UNAUTHORIZED);
        }
    }

    public static Long getTenantId() {
        try {
            return getLoginUser().getTenantId();
        } catch (Exception e) {
            throw new ServiceException("获取租户ID异常", HttpStatus.UNAUTHORIZED);
        }
    }

    /**
     * 获取用户
     **/
    public static LoginUser getLoginUser() {
        try {
            return (LoginUser) getAuthentication().getPrincipal();
        } catch (Exception e) {
            throw new ServiceException("获取用户信息异常", HttpStatus.UNAUTHORIZED);
        }
    }

    /**
     * 获取Authentication
     */
    public static Authentication getAuthentication() {
        return SecurityContextHolder.getContext().getAuthentication();
    }

    /**
     * 生成BCryptPasswordEncoder密码
     *
     * @param password 密码
     * @return 加密字符串
     */
    public static String encryptPassword(String password) {
        BCryptPasswordEncoder passwordEncoder = new BCryptPasswordEncoder();
        return passwordEncoder.encode(password);
    }

    /**
     * 判断密码是否相同
     *
     * @param rawPassword     真实密码
     * @param encodedPassword 加密后字符
     * @return 结果
     */
    public static boolean matchesPassword(String rawPassword, String encodedPassword) {
        BCryptPasswordEncoder passwordEncoder = new BCryptPasswordEncoder();
        return passwordEncoder.matches(rawPassword, encodedPassword);
    }

    /**
     * 是否为管理员
     *
     * @param userId 用户ID
     * @return 结果
     */
    public static boolean isAdmin(Long userId) {
        return userId != null && 1L == userId;
    }

    /**
     * 01的情况是租户管理员
     * @param userType
     * @param tenantId
     * @return
     */
    public static boolean isTenantAdmin(String userType, Long tenantId) {
        return tenantId != null && "01".equals(userType);
    }

    /**
     * 登录用户是否是租户管理员
     * @return
     */
    public static boolean isLoginUserTenantAdmin() {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        if (loginUser == null) {
            return false;
        } else {
            return loginUser.isTenantAdmin();
        }
    }


    /**
     * 获取当前登录用户的数据源信息
     *
     * @return
     */
    public static DataSource getDatasourceInfo() {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        return loginUser.getUser().getTenant().getDataSource();
    }

    /**
     * 获取数据源类型
     */
    public static String getDatabaseType() {
        try {
            return getLoginUser().getUser().getTenant().getDataSource().getDatabaseType();
        } catch (Exception e) {
            throw new TenantDataSource("未解析到当前请求中数据源信息", HttpStatus.ERRORWARN);
        }
    }

    /**
     * 获取数据源名称
     */
    public static String getDatabaseName() {
        try {
            return getLoginUser().getUser().getTenant().getDataSource().getDatabaseName();
        } catch (Exception e) {
            throw new ServiceException("获取获取数据名称异常", HttpStatus.UNAUTHORIZED);
        }
    }
}
